@@ -134,7 +134,6 @@ class MyApp(wx.App):
     class sensorProducer(threading.Thread):
         def __init__(self, _notify_window,portName,SERTIMEOUT,RECONNATTEMPTS,_nb):
 
-
             from Queue import Queue
             self.portName = portName
             self.RECONNATTEMPTS=RECONNATTEMPTS
@@ -144,8 +143,8 @@ class MyApp(wx.App):
             self._nb=_nb
             threading.Thread.__init__ ( self )
         
-        def initCommunication(self):
 
+        def initCommunication(self):
 
             #print self.portName
             #print self._notify_window
@@ -174,18 +173,23 @@ class MyApp(wx.App):
                     wx.PostEvent(self._notify_window, ResultEvent([i,0,""]))
             return "OK"
         
+
         def run(self):
 
-
             wx.PostEvent(self._notify_window, StatusEvent([0,1,"Connecting...."]))
-            self.initCommunication()
-            if self.port.State==0: #cant connect, exit thread
-              self.stop()
-              wx.PostEvent(self._notify_window, StatusEvent([666])) #signal apl, that communication was disconnected
-              wx.PostEvent(self._notify_window, StatusEvent([0,1,"Error cant connect..."]))
-              return None
-
-
+            try:
+               self.initCommunication()
+            except:
+               pass
+            try:
+               if self.port.State==0: #cant connect, exit thread
+                   self.stop()
+                   wx.PostEvent(self._notify_window, StatusEvent([666])) #signal apl, that communication was disconnected
+                   wx.PostEvent(self._notify_window, StatusEvent([0,1,"Error cant connect..."]))
+                   return None
+            except:
+               wx.PostEvent(self._notify_window, StatusEvent([0,1,"Disconnected"]))
+               return None
               
             wx.PostEvent(self._notify_window, StatusEvent([0,1,"Connected"]))
             os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"Connected to ECU"')
@@ -241,27 +245,40 @@ class MyApp(wx.App):
                  pass 
             self.stop()
 
+
         def off(self, id):
+
             if id >= 0 and id < len(self.active): 
                 self.active[id] = 0
             else:
                 debug("Invalid sensor id")
+
+
         def on(self, id):
+
             if id >= 0 and id < len(self.active): 
                 self.active[id] = 1
             else:
                 debug("Invalid sensor id")
 
         def all_off(self):
+
             for i in range(0, len(self.active)):
                 self.off(i)
+
+
         def all_on(self):
+
             for i in range(0, len(self.active)):
                 self.off(i)
                 
         def stop(self):
             if self.port != None: #if stop is called before any connection port is not defined (and not connected )
-              self.port.close()
+               try:
+                  self.port.close()
+               except:
+                  pass
+
             wx.PostEvent(self._notify_window, StatusEvent([0,1,"Disconnected"]))
             wx.PostEvent(self._notify_window, StatusEvent([2,1,"----"]))
   
@@ -278,9 +295,6 @@ class MyApp(wx.App):
 
         os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"Trying to connecting to ECU, wait some seconds..."')
 
-##
-##
-##
         def sensor_toggle(e):
             sel = e.m_itemIndex
             state = self.senprod.active[sel]
@@ -576,26 +590,40 @@ either version 2 of the License, or (at your option) any later version.
     def OnResult(self,event):
         self.sensors.SetStringItem(event.data[0], event.data[1], event.data[2])
     
+
     def OnStatus(self,event):
         if event.data[0] == 666: #signal, that connection falied
             self.sensor_control_off()
         else:
             self.status.SetStringItem(event.data[0], event.data[1], event.data[2])
     
+
     def OnTests(self,event):
         self.OBDTests.SetStringItem(event.data[0], event.data[1], event.data[2])
          
+
     def OnDebug(self,event):    
         self.TraceDebug(event.data[0],event.data[1])
     
+
     def OnDtc(self,event):
         if event.data == 0: #signal, that DTC was cleared
           self.dtc.DeleteAllItems()
         else:
           self.dtc.Append(event.data)
-    
+
+
     def OnDisconnect(self,event): #disconnect connection to ECU
         global btaddr
+        print "stop"
+
+        #w = sensorProduce()
+        #w.ws.PostEvent(self._notify_window, StatusEvent([0,1,"Disconnected"]))
+
+
+
+
+        time.sleep(0.1)
         self.btname = " "
         self.ThreadControl=666
         self.sensor_control_off()
@@ -607,9 +635,7 @@ either version 2 of the License, or (at your option) any later version.
         time.sleep(0.1)
         self.settingmenu.Enable(ID_RESET,False)
 
-
-
-        
+       
     def OpenPort(self,e):
         
         if self.senprod: # signal current producers to finish
@@ -696,6 +722,7 @@ either version 2 of the License, or (at your option) any later version.
         global found, btname, btaddr
         target_address = None
         number_of_tries = 2
+        btname=""
 
 # nome
 
@@ -714,33 +741,45 @@ either version 2 of the License, or (at your option) any later version.
              for btaddr in nearby_devices:
                btname = lookup_name(btaddr)
                print " -Found device: %s (%s)" % (btname, btaddr)
-               #if btname is not None and btname.startswith(target_name) == True:
-               if len(btname) > 2:
-                 found=1
-                 # Use the first device it finds that matches name
-                 print '    Found BT SPP dev'
-                 os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"Found a BT dev. Please wait"')
 
-                 target_address = btaddr
+               try:
+                  if len(btname) > 2:
+                    found=1
+                    # Use the first device it finds that matches name
+                    print '    Found BT SPP dev'
+                    os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"Found a BT dev. Please wait"')
 
+                    target_address = btaddr
+
+                    bus = dbus.SystemBus()
+                    manager = dbus.Interface(bus.get_object("org.bluez", "/"),"org.bluez.Manager")
+                    adapter = dbus.Interface(bus.get_object("org.bluez", manager.DefaultAdapter()),"org.bluez.Adapter")
+                    address = btaddr
+                    service = "spp"
+                    path = adapter.FindDevice(address)
+                    seriale = dbus.Interface(bus.get_object("org.bluez", path),"org.bluez.Serial")
+                    try:
+                        node = seriale.Disconnect(service)
+                    except:
+                        pass
+
+                    try:
+                        node = seriale.Connect(service)
+                    except:
+                        print "Can't open BT serial port"
+                        os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"Can\'t open BT serial port"')
+                        found=2
 
-                 bus = dbus.SystemBus()
-                 manager = dbus.Interface(bus.get_object("org.bluez", "/"),"org.bluez.Manager")
-                 adapter = dbus.Interface(bus.get_object("org.bluez", manager.DefaultAdapter()),"org.bluez.Adapter")
-                 address = btaddr
-                 service = "spp"
-                 path = adapter.FindDevice(address)
-                 seriale = dbus.Interface(bus.get_object("org.bluez", path),"org.bluez.Serial")
-                 try:
-                     node = seriale.Disconnect(service)
-                 except:
-                     pass
-                 node = seriale.Connect(service)
                  #print "Connected %s to %s" % (node, address)
                  #print "Press CTRL-C to disconnect"
                  #a = open("/dev/rfcomm0","rw")
-               else:
-                 print "NOT FOUND"
+                  else:
+                    print "NOT FOUND"
+               except:
+                    found=0
+                    print "error"
+                    os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"BT Error, please retry"')
+
              break
 
 #rfcomm bluetooth
@@ -828,7 +867,6 @@ either version 2 of the License, or (at your option) any later version.
         self.portName = ports
         self.btname = btname
 
-
 #       rb = wx.RadioBox(diag, id, "Choose Serial Port", choices = ports, style = wx.RA_SPECIFY_COLS, majorDimension = 20)
 
                 
@@ -873,33 +911,52 @@ either version 2 of the License, or (at your option) any later version.
         diag.SetSizer(sizer)
         diag.SetAutoLayout(True)
         sizer.Fit(diag)
-        r  = diag.ShowModal()
-        if r == wx.ID_OK:
+        if found==1:
+           r = diag.ShowModal()
+           if r == wx.ID_OK:
+              os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"OK, now select \'OBD-II\', then \'Connect to ECU\'"')
+              self.settingmenu.Enable(ID_RESET,True)
+              time.sleep(0.1)
+
+
             
             #create section
-            if self.config.sections()==[]:
-              self.config.add_section("pyOBD")
+              if self.config.sections()==[]:
+                 self.config.add_section("pyOBD")
             #set and save COMPORT
-#m            self.COMPORT = ports[rb.GetSelection()]
-            self.COMPORT = ports
-            self.config.set("pyOBD","COMPORT",self.COMPORT) 
+#m               self.COMPORT = ports[rb.GetSelection()]
+                 self.COMPORT = ports
+                 self.config.set("pyOBD","COMPORT",self.COMPORT) 
             
             #set and save SERTIMEOUT
-            self.SERTIMEOUT = int(timeoutCtrl.GetValue())
-            self.config.set("pyOBD","SERTIMEOUT",self.SERTIMEOUT)
-            self.status.SetStringItem(3,1,self.COMPORT); 
-            self.status.SetStringItem(4,1,self.btname); 
+                 self.SERTIMEOUT = int(timeoutCtrl.GetValue())
+                 self.config.set("pyOBD","SERTIMEOUT",self.SERTIMEOUT)
+                 self.status.SetStringItem(3,1,self.COMPORT); 
+                 self.status.SetStringItem(4,1,self.btname); 
             
             #set and save RECONNATTEMPTS
-            self.RECONNATTEMPTS = int(reconnectCtrl.GetValue())
-            self.config.set("pyOBD","RECONNATTEMPTS",self.RECONNATTEMPTS)
+                 self.RECONNATTEMPTS = int(reconnectCtrl.GetValue())
+                 self.config.set("pyOBD","RECONNATTEMPTS",self.RECONNATTEMPTS)
             
             #write configuration to cfg file
-            self.config.write(open(self.configfilepath, 'wb'))
+                 self.config.write(open(self.configfilepath, 'wb'))
 
-            self.settingmenu.Enable(ID_RESET,True)
+                 self.settingmenu.Enable(ID_RESET,True)
             #self.settingmenu.Enable(ID_DISCONNECT,True)
 
+        elif found==2:
+            time.sleep(0.7)
+            os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"BT dev. Found, but can\'t open serial port"')
+            time.sleep(0.8)
+            os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"BT dev. Found, but can\'t open serial port"')
+
+        else:
+            print "BlueTooth OBD device NOT detected"
+            time.sleep(0.7)
+            os.popen('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:"No BT dev. Found"')
+            time.sleep(0.7)
+
+
         
     def OnExit(self,e = None):
         import sys
